package com.smppw.analysis.domain.service.impl;

import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.common.pojo.IStrategy;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.*;
import com.smppw.core.IndicatorService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class BaseIndicatorServiceV2Impl implements BaseIndicatorServiceV2 {
    private final NavService navService;

    public BaseIndicatorServiceV2Impl(NavService navService) {
        this.navService = navService;
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> calcMultipleSecMultipleTimeRangeIndicator(CalcMultipleSecMultipleTimeRangeIndicatorReq req) {
        List<String> mainSecIdList = req.getMainSecIdList();
        Map<String, String> secBenchmarkIdMap = req.getSecBenchmarkIdMap();
        List<String> indexIdList = req.getIndexIdList();
        RaiseType raiseType = req.getRaiseType();
        IStrategy strategy = req.getStrategy();
        Visibility visibility = req.getVisibility();
        NavType navType = req.getNavType();
        String riskOfFreeId = req.getRiskOfFreeId();

        List<String> benchmarkIdList = new ArrayList<>();
        for (Map.Entry<String, String> secBenchmarkIdMapEntry : secBenchmarkIdMap.entrySet()) {
            String benchmarkId = secBenchmarkIdMapEntry.getValue();
            if (StrUtil.isNotEmpty(benchmarkId)) {
                benchmarkIdList.add(benchmarkId);
            }
        }
        CurveType curveType = CurveType.getCurveType(raiseType, strategy);
        if (StrUtil.isNotEmpty(riskOfFreeId)) {
            benchmarkIdList.add(riskOfFreeId);
        }
        Map<String, List<DateValue>> allNavMap = navService.getSecIdDateValueNavListMapFromRedisAndDB(mainSecIdList, benchmarkIdList, indexIdList,
                null, null, navType, visibility, curveType.getId(), strategy.getStrategyId(), new HashMap<>(), false);

        return IndicatorService.getInstance().calcMultipleSecMultipleTimeRangeIndicator(req, allNavMap);
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> getMultipleSecTrend(List<String> mainSecIdList, Map<String, String> secBenchmarkIdMap, List<String> indexIdList, DateIntervalDto dateIntervalDto, Frequency frequency, BigDecimal fixedIncome, BigDecimal initValue, RaiseType raiseType, IStrategy strategy, Visibility visibility, NavType navType, List<TrendType> trendTypeV2List, Boolean ifExtract, Integer userId) {
        List<String> benchmarkIdList = new ArrayList<>();
        for (Map.Entry<String, String> secBenchmarkIdMapEntry : secBenchmarkIdMap.entrySet()) {
            String benchmarkId = secBenchmarkIdMapEntry.getValue();
            if (StrUtil.isNotEmpty(benchmarkId)) {
                benchmarkIdList.add(benchmarkId);
            }
        }
        CurveType curveType = CurveType.getCurveType(raiseType, strategy);
        Map<String, List<DateValue>> allNavMap = navService.getSecIdDateValueNavListMapFromRedisAndDB(mainSecIdList, benchmarkIdList, indexIdList,
                null, null, navType, visibility, curveType.getId(), strategy.getStrategyId(), new HashMap<>(), false);

        return IndicatorService.getInstance().getMultipleSecTrend(mainSecIdList, secBenchmarkIdMap, indexIdList, dateIntervalDto, frequency, fixedIncome, initValue, trendTypeV2List, allNavMap);
    }
}
